# Zope imports
from AccessControl import Unauthorized

# zope imports
from zope.interface import directlyProvides
from zope.component import provideUtility

# test imports
from base import RatingTestCase

# iqpp.rating imports
from iqpp.rating.interfaces import IRateable
from iqpp.rating.interfaces import IRatingManager
from iqpp.rating.interfaces import IRatingDefinition    
from iqpp.rating.definition import RatingDefinition

class TestRate(RatingTestCase):

    def afterSetUp(self):
        self.setRoles(("Manager",))        
        self.portal.invokeFactory("Document", "document")

        self.d = self.portal.document        
        directlyProvides(self.d, IRateable)

        # create a definition
        sc = [("1", 1.0), ("2", 2.0), ("3", 3.0), ("4", 4.0),  ("5", 5.0),  ("6", 6.0)]
        score = RatingDefinition("Score", sc, "The Score for a Paper")
        provideUtility(score, IRatingDefinition, name='score')

    def testRate(self):
        rm = IRatingManager(self.portal.document)
        rm.rate(id="score", score="1", user="john")
        rm.rate(id="score", score="2", user="jane")
        rm.rate(id="score", score="3", user="ben")
                        
        rating = rm.getRating(id="score", user="john")
        self.assertEqual(rating, "1")

        rating = rm.getRating(id="score", user="jane")
        self.assertEqual(rating, "2")

        rating = rm.getRating(id="score", user="ben")
        self.assertEqual(rating, "3")

        ratings = rm.getRatings(id="score")
        self.assertEqual(len(ratings), 3)

        ratings = rm.getAmountOfRatings(id="score")
        self.assertEqual(ratings, 3)
        
        average = rm.getAverageRating(id="score")
        self.assertEqual(average, 2.0)

    def testNoRatings(self):
        rm = IRatingManager(self.portal.document)
        average = rm.getAverageRating(id="score")
        self.assertEqual(average, -1)
        
    def testWrongValue(self):
        rm = IRatingManager(self.portal.document)
        self.assertRaises(ValueError, rm.rate, id="score", score="42", user="john")

    def testWrongDefinition(self):
        rm = IRatingManager(self.portal.document)
        self.assertRaises(ValueError, rm.rate, id="wrong", score="1", user="john")

    def testChangeRating(self):
        rm = IRatingManager(self.portal.document)
        rm.rate(id="score", score="1", user="john")

        rating = rm.getRating(id="score", user="john")
        self.assertEqual(rating, "1")
        
        rm.rate(id="score", score="2", user="john")
        self.assertEqual(rating, "1")        

class TestAnonymousRating(RatingTestCase):

    def afterSetUp(self):
        self.setRoles(("Manager",))        
        self.portal.invokeFactory("Document", "document")

        self.d = self.portal.document        
        directlyProvides(self.d, IRateable)

        # create a definition
        sc = [("1", 1.0), ("2", 2.0), ("3", 3.0), ("4", 4.0),  ("5", 5.0),  ("6", 6.0)]
        score = RatingDefinition("Score", sc, "The Score for a Paper")
        provideUtility(score, IRatingDefinition, name='score')

    def testRate(self):
        rm = IRatingManager(self.portal.document)
        self.logout()
        
        rm.rate(id="score", user=None, score="1", ipaddress="127.0.0.1")
        rm.rate(id="score", user=None, score="2", ipaddress="127.0.0.2")
        rm.rate(id="score", user=None, score="3", ipaddress="127.0.0.3")

        self.login("test_user_1_")
        rating = rm.getRating(id="score")
        self.assertEqual(rating, 2.0)

        ratings = rm.getRatings(id="score")
        self.assertEqual(len(ratings), 1)

        ratings = rm.getAmountOfRatings(id="score")
        self.assertEqual(ratings, 3)
        
        average = rm.getAverageRating(id="score")
        self.assertEqual(average, 2.0)


class TestPermissions(RatingTestCase):

    def afterSetUp(self):
        self.setRoles(("Manager",))        
        self.portal.invokeFactory("Document", "document")

        self.d = self.portal.document        
        directlyProvides(self.d, IRateable)

        # create a definition
        sc = [("1", 1.0), ("2", 2.0), ("3", 3.0), ("4", 4.0),  ("5", 5.0),  ("6", 6.0)]
        score = RatingDefinition("Score", sc, "The Score for a Paper")
        provideUtility(score, IRatingDefinition, name='score')
    
        self.logout()
        
    def testRate(self):    
        rm = IRatingManager(self.portal.document)
        self.assertRaises(Unauthorized, rm.rate, id="score", score="1", user="john")
        
def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestRate))
    suite.addTest(makeSuite(TestAnonymousRating))    
    # suite.addTest(makeSuite(TestPermissions))        
    return suite
